@@ -55,16 +55,36 @@ public class FastByteOperations
         return -BestHolder.BEST.compare(b2, b1, s1, l1);
     }
 
+    public static int compareUnsigned(ByteBuffer b1, int s1, int l1, byte[] b2, int s2, int l2)
+    {
+        return BestHolder.BEST.compare(b1, s1, l1, b2, s2, l2);
+    }
+
+    public static int compareUnsigned(byte[] b1, int s1, int l1, ByteBuffer b2, int s2, int l2)
+    {
+        return -BestHolder.BEST.compare(b2, s2, l2, b1, s1, l1);
+    }
+
     public static int compareUnsigned(ByteBuffer b1, ByteBuffer b2)
     {
         return BestHolder.BEST.compare(b1, b2);
     }
 
+    public static void copy(byte[] src, int srcPosition, byte[] trg, int trgPosition, int length)
+    {
+        BestHolder.BEST.copy(src, srcPosition, trg, trgPosition, length);
+    }
+
     public static void copy(ByteBuffer src, int srcPosition, byte[] trg, int trgPosition, int length)
     {
         BestHolder.BEST.copy(src, srcPosition, trg, trgPosition, length);
     }
 
+    public static void copy(byte[] src, int srcPosition, ByteBuffer trg, int trgPosition, int length)
+    {
+        BestHolder.BEST.copy(src, srcPosition, trg, trgPosition, length);
+    }
+
     public static void copy(ByteBuffer src, int srcPosition, ByteBuffer trg, int trgPosition, int length)
     {
         BestHolder.BEST.copy(src, srcPosition, trg, trgPosition, length);
@@ -77,10 +97,16 @@ public class FastByteOperations
 
         abstract public int compare(ByteBuffer buffer1, byte[] buffer2, int offset2, int length2);
 
+        abstract public int compare(ByteBuffer buffer1, int offset1, int length1, byte[] buffer2, int offset2, int length2);
+
         abstract public int compare(ByteBuffer buffer1, ByteBuffer buffer2);
 
+        abstract public void copy(byte[] src, int srcPosition, byte[] trg, int trgPosition, int length);
+
         abstract public void copy(ByteBuffer src, int srcPosition, byte[] trg, int trgPosition, int length);
 
+        abstract public void copy(byte[] src, int srcPosition, ByteBuffer trg, int trgPosition, int length);
+
         abstract public void copy(ByteBuffer src, int srcPosition, ByteBuffer trg, int trgPosition, int length);
     }
 
@@ -186,26 +212,25 @@ public class FastByteOperations
         }
 
         public int compare(ByteBuffer buffer1, byte[] buffer2, int offset2, int length2)
+        {
+            return compare(buffer1, buffer1.position(), buffer1.remaining(), buffer2, offset2, length2);
+        }
+
+        public int compare(ByteBuffer buffer1, int position1, int length1, byte[] buffer2, int offset2, int length2)
         {
             Object obj1;
             long offset1;
             if (buffer1.hasArray())
             {
                 obj1 = buffer1.array();
-                offset1 = BYTE_ARRAY_BASE_OFFSET + buffer1.arrayOffset();
+                offset1 = BYTE_ARRAY_BASE_OFFSET + buffer1.arrayOffset() + position1;
             }
             else
             {
                 obj1 = null;
-                offset1 = theUnsafe.getLong(buffer1, DIRECT_BUFFER_ADDRESS_OFFSET);
-            }
-            int length1;
-            {
-                int position = buffer1.position();
-                int limit = buffer1.limit();
-                length1 = limit - position;
-                offset1 += position;
+                offset1 = theUnsafe.getLong(buffer1, DIRECT_BUFFER_ADDRESS_OFFSET) + position1;
             }
+
             return compareTo(obj1, offset1, length1, buffer2, BYTE_ARRAY_BASE_OFFSET + offset2, length2);
         }
 
@@ -214,6 +239,11 @@ public class FastByteOperations
             return compareTo(buffer1, buffer2);
         }
 
+        public void copy(byte[] src, int srcPosition, byte[] trg, int trgPosition, int length)
+        {
+            System.arraycopy(src, srcPosition, trg, trgPosition, length);
+        }
+
         public void copy(ByteBuffer src, int srcPosition, byte[] trg, int trgPosition, int length)
         {
             if (src.hasArray())
@@ -222,6 +252,14 @@ public class FastByteOperations
                 copy(null, srcPosition + theUnsafe.getLong(src, DIRECT_BUFFER_ADDRESS_OFFSET), trg, trgPosition, length);
         }
 
+        public void copy(byte[] src, int srcPosition, ByteBuffer trg, int trgPosition, int length)
+        {
+            if (trg.hasArray())
+                System.arraycopy(src, srcPosition, trg.array(), trg.arrayOffset() + trgPosition, length);
+            else
+                copy(null, srcPosition + theUnsafe.getLong(src, Unsafe.ARRAY_BYTE_BASE_OFFSET), trg, trgPosition, length);
+        }
+
         public void copy(ByteBuffer srcBuf, int srcPosition, ByteBuffer trgBuf, int trgPosition, int length)
         {
             Object src;
@@ -397,11 +435,28 @@ public class FastByteOperations
             return length1 - length2;
         }
 
+        public int compare(ByteBuffer buffer1, int position1, int length1, byte[] buffer2, int offset2, int length2)
+        {
+            if (buffer1.hasArray())
+                return compare(buffer1.array(), buffer1.arrayOffset() + position1, length1, buffer2, offset2, length2);
+
+            if (position1 != buffer1.position())
+            {
+                buffer1 = buffer1.duplicate();
+                buffer1.position(position1);
+            }
+
+            return compare(buffer1, ByteBuffer.wrap(buffer2, offset2, length2));
+        }
+
         public int compare(ByteBuffer buffer1, byte[] buffer2, int offset2, int length2)
         {
             if (buffer1.hasArray())
+            {
                 return compare(buffer1.array(), buffer1.arrayOffset() + buffer1.position(), buffer1.remaining(),
                                buffer2, offset2, length2);
+            }
+
             return compare(buffer1, ByteBuffer.wrap(buffer2, offset2, length2));
         }
 
@@ -421,6 +476,11 @@ public class FastByteOperations
             return buffer1.remaining() - buffer2.remaining();
         }
 
+        public void copy(byte[] src, int srcPosition, byte[] trg, int trgPosition, int length)
+        {
+            System.arraycopy(src, srcPosition, trg, trgPosition, length);
+        }
+
         public void copy(ByteBuffer src, int srcPosition, byte[] trg, int trgPosition, int length)
         {
             if (src.hasArray())
@@ -433,6 +493,16 @@ public class FastByteOperations
             src.get(trg, trgPosition, length);
         }
 
+        public void copy(byte[] src, int srcPosition, ByteBuffer trg, int trgPosition, int length)
+        {
+            if (trg.hasArray())
+            {
+                System.arraycopy(src, srcPosition, trg.array(), trg.arrayOffset() + trgPosition, length);
+                return;
+            }
+            trg.duplicate().put(src, srcPosition, length);
+        }
+
         public void copy(ByteBuffer src, int srcPosition, ByteBuffer trg, int trgPosition, int length)
         {
             if (src.hasArray() && trg.hasArray())
