@@ -28,17 +28,17 @@ import java.util.TreeMap;
 import java.util.UUID;
 import java.util.function.Function;
 
-import com.vdurmont.semver4j.Semver;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import com.vdurmont.semver4j.Semver;
 import org.apache.cassandra.distributed.api.Feature;
 import org.apache.cassandra.distributed.api.IInstanceConfig;
 import org.apache.cassandra.distributed.shared.NetworkTopology;
-import org.apache.cassandra.distributed.shared.Shared;
 import org.apache.cassandra.distributed.upgrade.UpgradeTestBase;
 import org.apache.cassandra.locator.InetAddressAndPort;
 import org.apache.cassandra.locator.SimpleSeedProvider;
+import org.apache.cassandra.utils.Shared;
 
 @Shared
 public class InstanceConfig implements IInstanceConfig
@@ -47,12 +47,15 @@ public class InstanceConfig implements IInstanceConfig
     private static final Logger logger = LoggerFactory.getLogger(InstanceConfig.class);
 
     public final int num;
+    private final int jmxPort;
+
     public int num() { return num; }
 
     private final NetworkTopology networkTopology;
     public NetworkTopology networkTopology() { return networkTopology; }
 
-    public final UUID hostId;
+    private volatile UUID hostId;
+    public void setHostId(UUID hostId) { this.hostId = hostId; }
     public UUID hostId() { return hostId; }
     private final Map<String, Object> params = new TreeMap<>();
     private final Map<String, Object> dtestParams = new TreeMap<>();
@@ -76,7 +79,8 @@ public class InstanceConfig implements IInstanceConfig
                            String cdc_raw_directory,
                            String initial_token,
                            int storage_port,
-                           int native_transport_port)
+                           int native_transport_port,
+                           int jmx_port)
     {
         this.num = num;
         this.networkTopology = networkTopology;
@@ -117,6 +121,7 @@ public class InstanceConfig implements IInstanceConfig
                 // legacy parameters
                 .forceSet("commitlog_sync_batch_window_in_ms", 1.0);
         this.featureFlags = EnumSet.noneOf(Feature.class);
+        this.jmxPort = jmx_port;
     }
 
     private InstanceConfig(InstanceConfig copy)
@@ -128,6 +133,7 @@ public class InstanceConfig implements IInstanceConfig
         this.hostId = copy.hostId;
         this.featureFlags = copy.featureFlags;
         this.broadcastAddressAndPort = copy.broadcastAddressAndPort;
+        this.jmxPort = copy.jmxPort;
     }
 
 
@@ -173,6 +179,12 @@ public class InstanceConfig implements IInstanceConfig
         return networkTopology().localDC(broadcastAddress());
     }
 
+    @Override
+    public int jmxPort()
+    {
+        return this.jmxPort;
+    }
+
     public InstanceConfig with(Feature featureFlag)
     {
         featureFlags.add(featureFlag);
@@ -199,7 +211,7 @@ public class InstanceConfig implements IInstanceConfig
         return this;
     }
 
-    private InstanceConfig forceSet(String fieldName, Object value)
+    public InstanceConfig forceSet(String fieldName, Object value)
     {
         if (value == null)
             value = NULL;
@@ -270,7 +282,8 @@ public class InstanceConfig implements IInstanceConfig
                                   String.format("%s/node%d/cdc", root, nodeNum),
                                   token,
                                   provisionStrategy.storagePort(nodeNum),
-                                  provisionStrategy.nativeTransportPort(nodeNum));
+                                  provisionStrategy.nativeTransportPort(nodeNum),
+                                  provisionStrategy.jmxPort(nodeNum));
     }
 
     private static String[] datadirs(int datadirCount, File root, int nodeNum)
