@@ -24,7 +24,6 @@ import org.junit.Test;
 
 import org.apache.cassandra.cql3.CQLTester;
 import org.apache.cassandra.dht.ByteOrderedPartitioner;
-import org.apache.cassandra.exceptions.*;
 import org.apache.cassandra.service.StorageService;
 
 public class UserTypesTest extends CQLTester
@@ -110,9 +109,9 @@ public class UserTypesTest extends CQLTester
         String myType = KEYSPACE + '.' + typename;
 
         // non-frozen UDTs in a table PK
-        assertInvalidMessage("Invalid non-frozen user-defined type for PRIMARY KEY component k",
+        assertInvalidMessage("Invalid non-frozen user-defined type \"" + myType + "\" for PRIMARY KEY column 'k'",
                 "CREATE TABLE " + KEYSPACE + ".wrong (k " + myType + " PRIMARY KEY , v int)");
-        assertInvalidMessage("Invalid non-frozen user-defined type for PRIMARY KEY component k2",
+        assertInvalidMessage("Invalid non-frozen user-defined type \"" + myType + "\" for PRIMARY KEY column 'k2'",
                 "CREATE TABLE " + KEYSPACE + ".wrong (k1 int, k2 " + myType + ", v int, PRIMARY KEY (k1, k2))");
 
         // non-frozen UDTs in a collection
@@ -133,10 +132,6 @@ public class UserTypesTest extends CQLTester
         assertInvalidMessage("A user type cannot contain non-frozen UDTs",
                 "CREATE TYPE " + KEYSPACE + ".wrong (a int, b " + myType + ")");
 
-        String ut1 = createType("CREATE TYPE %s (a int)");
-        assertInvalidMessage("A user type cannot contain non-frozen UDTs",
-                             "ALTER TYPE " + KEYSPACE + "." + ut1 + " ADD b " + myType);
-
         // referencing a UDT in another keyspace
         assertInvalidMessage("Statement on keyspace " + KEYSPACE + " cannot refer to a user type in keyspace otherkeyspace;" +
                              " user types can only be used in the keyspace they are defined in",
@@ -481,20 +476,6 @@ public class UserTypesTest extends CQLTester
         execute("SELECT addresses FROM %s WHERE id = ? ", userID_1);
     }
 
-    @Test
-    public void testCreateTypeWithUndesiredFieldType() throws Throwable
-    {
-        String typeName = createTypeName();
-        assertInvalidMessage("A user type cannot contain counters", "CREATE TYPE " + typeWithKs(typeName) + " (f counter)");
-    }
-
-    @Test
-    public void testAlterTypeWithUndesiredFieldType() throws Throwable
-    {
-        String typeName = createType("CREATE TYPE %s (a int)");
-        assertInvalidMessage("A user type cannot contain counters", "ALTER TYPE " + typeWithKs(typeName) + " ADD f counter");
-    }
-
     /**
      * Test user type test that does a little more nesting,
      * migrated from cql_tests.py:TestCQL.more_user_types_test()
@@ -639,7 +620,7 @@ public class UserTypesTest extends CQLTester
     private void assertInvalidAlterDropStatements(String t) throws Throwable
     {
         assertInvalidMessage("Cannot alter user type " + typeWithKs(t), "ALTER TYPE " + typeWithKs(t) + " RENAME foo TO bar;");
-        assertInvalidMessage("Cannot drop user type " + typeWithKs(t), "DROP TYPE " + typeWithKs(t) + ';');
+        assertInvalidMessage("Cannot drop user type '" + typeWithKs(t), "DROP TYPE " + typeWithKs(t) + ';');
     }
 
     @Test
@@ -897,25 +878,6 @@ public class UserTypesTest extends CQLTester
                        row(1, 1,set(userType("a", 1), userType("a", 1, "b", 1), userType("a", 1, "b", 2), userType("a", 2), userType("a", 2, "b", 1)), 2));
     }
 
-    @Test(expected = SyntaxException.class)
-    public void emptyTypeNameTest() throws Throwable
-    {
-        execute("CREATE TYPE \"\" (a int, b int)");
-    }
-
-    @Test(expected = SyntaxException.class)
-    public void emptyFieldNameTest() throws Throwable
-    {
-        execute("CREATE TYPE mytype (\"\" int, b int)");
-    }
-
-    @Test(expected = SyntaxException.class)
-    public void renameColumnToEmpty() throws Throwable
-    {
-        String typeName = createType("CREATE TYPE %s (a int, b int)");
-        execute(String.format("ALTER TYPE %s.%s RENAME b TO \"\"", keyspace(), typeName));
-    }
-
     private String typeWithKs(String type1)
     {
         return keyspace() + '.' + type1;
