@@ -26,11 +26,12 @@ import java.util.regex.Pattern;
 
 import org.junit.Test;
 
+import org.assertj.core.api.Assertions;
+
 import static org.junit.Assert.assertEquals;
 
 public class TableStatsPrinterTest extends TableStatsTestBase
 {
-
     public static final String expectedDefaultTable1Output =
         "\tTable: %s\n" +
         "\tSSTable count: 60000\n" +
@@ -99,7 +100,7 @@ public class TableStatsPrinterTest extends TableStatsTestBase
         "\tCompacted partition maximum bytes: 30\n" +
         "\tCompacted partition mean bytes: 4\n" +
         "\tAverage live cells per slice (last five minutes): 4.01\n" +
-        "\tMaximum live cells per slice (last five minutes): 5\n" + 
+        "\tMaximum live cells per slice (last five minutes): 5\n" +
         "\tAverage tombstones per slice (last five minutes): 4.001\n" +
         "\tMaximum tombstones per slice (last five minutes): 2\n" +
         "\tDropped Mutations: 222\n" +
@@ -381,6 +382,130 @@ public class TableStatsPrinterTest extends TableStatsTestBase
         }
     }
 
+    @Test
+    public void testJsonPrinter() throws Exception
+    {
+        TestTableStatsHolder holder = new TestTableStatsHolder(testKeyspaces.subList(2, 3), "", 0); // kesypace3
+        StatsPrinter<StatsHolder> printer = TableStatsPrinter.from("json", false);
+        try (ByteArrayOutputStream byteStream = new ByteArrayOutputStream())
+        {
+            printer.print(holder, new PrintStream(byteStream));
+            Assertions.assertThat(byteStream.toString())
+                      .isEqualTo("{\n" +
+                                 "  \"keyspace3\" : {\n" +
+                                 "    \"write_latency_ms\" : \"NaN\",\n" +
+                                 "    \"tables\" : {\n" +
+                                 "      \"table6\" : {\n" +
+                                 "        \"average_tombstones_per_slice_last_five_minutes\" : 6.0,\n" +
+                                 "        \"bloom_filter_off_heap_memory_used\" : \"667408\",\n" +
+                                 "        \"bytes_pending_repair\" : 0,\n" +
+                                 "        \"memtable_switch_count\" : 6,\n" +
+                                 "        \"maximum_tombstones_per_slice_last_five_minutes\" : 6,\n" +
+                                 "        \"memtable_cell_count\" : 6666,\n" +
+                                 "        \"memtable_data_size\" : \"1000000\",\n" +
+                                 "        \"average_live_cells_per_slice_last_five_minutes\" : 5.0,\n" +
+                                 "        \"local_read_latency_ms\" : \"1.000\",\n" +
+                                 "        \"sstable_count\" : 1000,\n" +
+                                 "        \"local_write_latency_ms\" : \"0.500\",\n" +
+                                 "        \"pending_flushes\" : 66,\n" +
+                                 "        \"compacted_partition_minimum_bytes\" : 6,\n" +
+                                 "        \"local_read_count\" : 5,\n" +
+                                 "        \"sstable_compression_ratio\" : 0.68,\n" +
+                                 "        \"dropped_mutations\" : \"666666\",\n" +
+                                 "        \"bloom_filter_false_positives\" : 400,\n" +
+                                 "        \"off_heap_memory_used_total\" : \"162470810\",\n" +
+                                 "        \"memtable_off_heap_memory_used\" : \"161803398\",\n" +
+                                 "        \"index_summary_off_heap_memory_used\" : \"3\",\n" +
+                                 "        \"bloom_filter_space_used\" : \"101112\",\n" +
+                                 "        \"sstables_in_each_level\" : [ ],\n" +
+                                 "        \"compacted_partition_maximum_bytes\" : 20,\n" +
+                                 "        \"space_used_total\" : \"0\",\n" +
+                                 "        \"local_write_count\" : 0,\n" +
+                                 "        \"droppable_tombstone_ratio\" : \"0.66667\",\n" +
+                                 "        \"compression_metadata_off_heap_memory_used\" : \"1\",\n" +
+                                 "        \"number_of_partitions_estimate\" : 6,\n" +
+                                 "        \"bytes_repaired\" : 0,\n" +
+                                 "        \"maximum_live_cells_per_slice_last_five_minutes\" : 2,\n" +
+                                 "        \"space_used_live\" : \"666666\",\n" +
+                                 "        \"compacted_partition_mean_bytes\" : 3,\n" +
+                                 "        \"bloom_filter_false_ratio\" : \"0.03000\",\n" +
+                                 "        \"old_sstable_count\" : 0,\n" +
+                                 "        \"bytes_unrepaired\" : 0,\n" +
+                                 "        \"percent_repaired\" : 0.0,\n" +
+                                 "        \"space_used_by_snapshots_total\" : \"0\"\n" +
+                                 "      }\n" +
+                                 "    },\n" +
+                                 "    \"read_latency_ms\" : 0.0,\n" +
+                                 "    \"pending_flushes\" : 66,\n" +
+                                 "    \"write_count\" : 0,\n" +
+                                 "    \"read_latency\" : 0.0,\n" +
+                                 "    \"read_count\" : 5\n" +
+                                 "  },\n" +
+                                 "  \"total_number_of_tables\" : 0\n" +
+                                 "}\n");
+        }
+    }
+
+    @Test
+    public void testYamlPrinter() throws Exception
+    {
+        TestTableStatsHolder holder = new TestTableStatsHolder(testKeyspaces.subList(2, 3), "", 0); // kesypace3
+        StatsPrinter<StatsHolder> printer = TableStatsPrinter.from("yaml", false);
+        try (ByteArrayOutputStream byteStream = new ByteArrayOutputStream())
+        {
+            printer.print(holder, new PrintStream(byteStream));
+            Assertions.assertThat(byteStream.toString())
+                      .isEqualTo("keyspace3:\n" +
+                                 "  write_latency_ms: .NaN\n" +
+                                 "  tables:\n" +
+                                 "    table6:\n" +
+                                 "      average_tombstones_per_slice_last_five_minutes: 6.0\n" +
+                                 "      bloom_filter_off_heap_memory_used: '667408'\n" +
+                                 "      bytes_pending_repair: 0\n" +
+                                 "      memtable_switch_count: 6\n" +
+                                 "      maximum_tombstones_per_slice_last_five_minutes: 6\n" +
+                                 "      memtable_cell_count: 6666\n" +
+                                 "      memtable_data_size: '1000000'\n" +
+                                 "      average_live_cells_per_slice_last_five_minutes: 5.0\n" +
+                                 "      local_read_latency_ms: '1.000'\n" +
+                                 "      sstable_count: 1000\n" +
+                                 "      local_write_latency_ms: '0.500'\n" +
+                                 "      pending_flushes: 66\n" +
+                                 "      compacted_partition_minimum_bytes: 6\n" +
+                                 "      local_read_count: 5\n" +
+                                 "      sstable_compression_ratio: 0.68\n" +
+                                 "      dropped_mutations: '666666'\n" +
+                                 "      bloom_filter_false_positives: 400\n" +
+                                 "      off_heap_memory_used_total: '162470810'\n" +
+                                 "      memtable_off_heap_memory_used: '161803398'\n" +
+                                 "      index_summary_off_heap_memory_used: '3'\n" +
+                                 "      bloom_filter_space_used: '101112'\n" +
+                                 "      sstables_in_each_level: []\n" +
+                                 "      compacted_partition_maximum_bytes: 20\n" +
+                                 "      space_used_total: '0'\n" +
+                                 "      local_write_count: 0\n" +
+                                 "      droppable_tombstone_ratio: '0.66667'\n" +
+                                 "      compression_metadata_off_heap_memory_used: '1'\n" +
+                                 "      number_of_partitions_estimate: 6\n" +
+                                 "      bytes_repaired: 0\n" +
+                                 "      maximum_live_cells_per_slice_last_five_minutes: 2\n" +
+                                 "      space_used_live: '666666'\n" +
+                                 "      compacted_partition_mean_bytes: 3\n" +
+                                 "      bloom_filter_false_ratio: '0.03000'\n" +
+                                 "      old_sstable_count: 0\n" +
+                                 "      bytes_unrepaired: 0\n" +
+                                 "      percent_repaired: 0.0\n" +
+                                 "      space_used_by_snapshots_total: '0'\n" +
+                                 "  read_latency_ms: 0.0\n" +
+                                 "  pending_flushes: 66\n" +
+                                 "  write_count: 0\n" +
+                                 "  read_latency: 0.0\n" +
+                                 "  read_count: 5\n" +
+                                 "total_number_of_tables: 0\n" +
+                                 "\n");
+        }
+    }
+
     /**
      * A test version of TableStatsHolder to hold a test vector instead of gathering stats from a live cluster.
      */
